import 'dart:developer';

import 'package:flutter/material.dart';
import 'package:hub/actions/actions.dart';
import 'package:hub/app_state.dart';
import 'package:hub/backend/api_requests/api_calls.dart';
import 'package:hub/backend/api_requests/api_manager.dart';
import 'package:hub/components/templates_components/card_item_template_component/card_item_template_component_widget.dart';
import 'package:hub/components/templates_components/details_component/details_component_action.dart';
import 'package:hub/flutter_flow/flutter_flow_theme.dart';
import 'package:hub/flutter_flow/flutter_flow_util.dart';
import 'package:hub/flutter_flow/internationalization.dart';
import 'package:hub/pages/schedule_complete_visit_page/schedule_complete_visit_page_model.dart';

class VisitHistoryWidget extends StatefulWidget {
  VisitHistoryWidget({
    Key? key,
  }) : super(key: key);

  @override
  _VisitHistoryWidgetState createState() => _VisitHistoryWidgetState();
}

class _VisitHistoryWidgetState extends State<VisitHistoryWidget> {
  List<dynamic> visitaWrap = [];
  var pageNumber = 1;
  late Future<void> visitFuture;

  @override
  void initState() {
    super.initState();
    visitFuture = _fetchVisits();
  }

  Future<ApiCallResponse?> _fetchVisits() async {
    try {
      var response = await ScheduleCompleteVisitPageModel().visitHistory(
        requestFn: () => PhpGroup.getVisitsCall.call(
          devUUID: AppState().devUUID,
          userUUID: AppState().userUUID,
          cliID: AppState().cliUUID,
          atividade: 'getVisitas',
          pageSize: 10,
          pageNumber: pageNumber,
        ),
      );

      var newVisits = response.jsonBody['visitas']
          as List<dynamic>?; // Ajuste conforme a estrutura da resposta
      if (newVisits != null && newVisits.isNotEmpty) {
        safeSetState(() {
          visitaWrap.addAll(newVisits);
        });
        return response;
      } else {
        log('No new visits found');
        return null;
      }
    } catch (err) {
      log('Erro ao carregar mais visitas: $err');
    }
  }

  void _loadMoreVisits() {
    pageNumber++;
    visitFuture = _fetchVisits();
  }

  @override
  Widget build(BuildContext context) {
    return FutureBuilder<void>(
      future: visitFuture,
      builder: (context, snapshot) {
        if (snapshot.connectionState == ConnectionState.waiting &&
            visitaWrap.isEmpty) {
          return Center(
            child: CircularProgressIndicator(
              valueColor: AlwaysStoppedAnimation<Color>(
                FlutterFlowTheme.of(context).primary,
              ),
            ),
          );
        } else if (snapshot.hasError) {
          return Center(child: Text('Error: ${snapshot.error}'));
        } else if (visitaWrap.isEmpty) {
          return Center(
            child: Text(
              FFLocalizations.of(context).getVariableText(
                ptText: 'Nenhum visitante foi encontrado',
                enText: 'No visitors found',
              ),
            ),
          );
        }

        return ListView.builder(
          itemCount: visitaWrap.length + 1,
          shrinkWrap: true,
          physics: const BouncingScrollPhysics(),
          itemBuilder: (context, index) {
            if (index == visitaWrap.length) {
              _loadMoreVisits();
              return Center(
                child: CircularProgressIndicator(
                  valueColor: AlwaysStoppedAnimation<Color>(
                    FlutterFlowTheme.of(context).primary,
                  ),
                ),
              );
            }

            final visitaWrapItem = visitaWrap[index];

            return CardItemTemplateComponentWidget(
              imagePath:
                  'https://freaccess.com.br/freaccess/getImage.php?devUUID=${AppState().devUUID}&userUUID=${AppState().userUUID}&cliID=${AppState().cliUUID}&atividade=getFoto&Documento=${visitaWrapItem['VTE_DOCUMENTO'] ?? ''}&tipo=E',
              labelsHashMap: {
                'Nome:': visitaWrapItem['VTE_NOME'] ?? '',
                'Inicio:': visitaWrapItem['VAW_DTINICIO'] ?? '',
                'Fim:': visitaWrapItem['VAW_DTFIM'] ?? '',
              },
              statusHashMap: [
                if (getStatus(visitaWrapItem['VAW_STATUS']) == status.active)
                  {
                    FFLocalizations.of(context).getVariableText(
                      ptText: 'Ativo',
                      enText: 'Active',
                    ): FlutterFlowTheme.of(context).warning,
                  },
                if (getStatus(visitaWrapItem['VAW_STATUS']) == status.finished)
                  {
                    FFLocalizations.of(context).getVariableText(
                      ptText: 'Finalizado',
                      enText: 'Finished',
                    ): FlutterFlowTheme.of(context).success,
                  },
                if (getStatus(visitaWrapItem['VAW_STATUS']) == status.unknown)
                  {
                    FFLocalizations.of(context).getVariableText(
                      ptText: 'Desconhecido',
                      enText: 'Unknown',
                    ): FlutterFlowTheme.of(context).alternate,
                  },
                if (getStatus(visitaWrapItem['VAW_STATUS']) == status.canceled)
                  {
                    FFLocalizations.of(context).getVariableText(
                      ptText: 'Cancelado',
                      enText: 'Canceled',
                    ): FlutterFlowTheme.of(context).error,
                  },
                if (getStatus(visitaWrapItem['VAW_STATUS']) == status.blocked)
                  {
                    FFLocalizations.of(context).getVariableText(
                      ptText: 'Bloqueado',
                      enText: 'Blocked',
                    ): FlutterFlowTheme.of(context).error,
                  },
                if (getStatus(visitaWrapItem['VAW_STATUS']) == status.inactive)
                  {
                    FFLocalizations.of(context).getVariableText(
                      ptText: 'Inativo',
                      enText: 'Inactive',
                    ): FlutterFlowTheme.of(context).error,
                  },
              ],
              onTapCardItemAction: () async {
                await showDialog(
                  useSafeArea: true,
                  context: context,
                  builder: (context) {
                    return Dialog(
                      alignment: Alignment.center,
                      child: buildDetails(
                        visitaWrapItem,
                        context,
                        changeStatusAction,
                      ),
                    );
                  },
                ).whenComplete(() {
                  // updateVisitFuture();
                  _fetchVisits().then((response) {
                    safeSetState(() {
                      visitaWrap = PhpGroup.getVisitsCall
                              .visitasList(response!.jsonBody)
                              ?.toList() ??
                          [];
                      visitFuture = Future.value(response);
                    });
                  });
                }).catchError((err, stack) {});
              },
            );
          },
        );
      },
    );
  }
}
